@@ -19,6 +19,12 @@ except Exception as e:
     _STACK_UNVAILABLE_REASON = str(e)
     _STACK_POSSIBLE = False
 
+try:
+    from aioisotp.sync import SyncISOTPNetwork
+    _AISOTP_POSSIBLE = True
+except Exception as e:
+    _AISOTP_POSSIBLE = False
+
 class TestIsoTPSocketConnection(UdsTest):
 
     def setUp(self):
@@ -106,7 +112,6 @@ class TestSocketConnection(UdsTest):
                 payload2 = conn2.wait_frame(timeout=1, exception=True)
                 self.assertEqual(payload1, payload2)
 
-
 class TestQueueConnection(UdsTest):
     def setUp(self):
         self.conn = QueueConnection(name='unittest')
@@ -208,3 +213,30 @@ class TestPythonIsoTpConnection(UdsTest):
         self.conn.close()
         self.vcan0_bus.shutdown()
 
+@unittest.skipIf(_AISOTP_POSSIBLE == False, "aisotp module is not present.")
+class TestSyncAioIsotpConnection(UdsTest):
+
+    def test_open(self):
+        conn0 = SyncAioIsotpConnection(interface="virtual", channel=0, bitrate=500000, rx_id=0x123, tx_id=0x456, name="unittest")
+
+        conn0.open()
+        self.assertTrue(conn0.is_open())
+        conn0.close()
+        self.assertFalse(conn0.is_open())
+
+        conn0.open()
+        self.assertTrue(conn0.is_open())
+        conn0.close()
+        self.assertFalse(conn0.is_open())
+
+    def test_transmit(self):
+        conn0 = SyncAioIsotpConnection(interface="virtual", channel=0, bitrate=500000, rx_id=0x123, tx_id=0x456, name="unittest")
+        conn1 = SyncAioIsotpConnection(interface="virtual", channel=0, bitrate=500000, rx_id=0x456, tx_id=0x123, name="unittest")
+    
+        with conn0.open():
+            with conn1.open():
+                tx_data = bytes([i for i in range(256)])
+                conn0.send(tx_data)
+
+                rx_data = conn1.wait_frame()
+                self.assertTrue(tx_data == rx_data)
