@@ -5,34 +5,38 @@ import logging
 import binascii
 import sys
 from abc import ABC, abstractmethod
-import functools
 import time
 
 try:
-    import can
+    import can  # type:ignore
     _import_can_err = None
 except Exception as e:
     _import_can_err = e
 
 try:
-    import isotp
+    import isotp    # type:ignore
     _import_isotp_err = None
 except Exception as e:
     _import_isotp_err = e
 
 try:
     from udsoncan.j2534 import J2534
-    from udsoncan.j2534 import Protocol_ID
+    from udsoncan.j2534 import Protocol_ID, Ioctl_Flags, Ioctl_ID
     _import_j2534_err = None
 except Exception as e:
     _import_j2534_err = e
 
-
+try:
+    from aioisotp.sync import SyncISOTPNetwork  # type:ignore
+    _import_aioisotp_err = None
+except Exception as e:
+    _import_aioisotp_err = e
 
 from udsoncan.Request import Request
 from udsoncan.Response import Response
 from udsoncan.exceptions import TimeoutException
 
+
 class BaseConnection(ABC):
 
     def __init__(self, name=None):
@@ -53,11 +57,11 @@ class BaseConnection(ABC):
         """
 
         if isinstance(data, Request) or isinstance(data, Response):
-            payload = data.get_payload()  
-        else :
+            payload = data.get_payload()
+        else:
             payload = data
 
-        self.logger.debug('Sending %d bytes : [%s]' % (len(payload), binascii.hexlify(payload) ))
+        self.logger.debug('Sending %d bytes : [%s]' % (len(payload), binascii.hexlify(payload)))
         self.specific_send(payload)
 
     def wait_frame(self, timeout=2, exception=False):
@@ -84,7 +88,7 @@ class BaseConnection(ABC):
                 frame = None
 
         if frame is not None:
-            self.logger.debug('Received %d bytes : [%s]' % (len(frame), binascii.hexlify(frame) ))
+            self.logger.debug('Received %d bytes : [%s]' % (len(frame), binascii.hexlify(frame)))
         return frame
 
     def __enter__(self):
@@ -127,7 +131,7 @@ class BaseConnection(ABC):
 
         :returns: None
         """
-        pass	
+        pass
 
     @abstractmethod
     def empty_rxqueue(self):
@@ -137,6 +141,14 @@ class BaseConnection(ABC):
         """
         pass
 
+    @abstractmethod
+    def is_open(self) -> bool:
+        """ Tells if the connection is open.
+
+        :returns: bool
+        """
+        pass
+
     def __exit__(self, type, value, traceback):
         pass
 
@@ -153,6 +165,7 @@ class SocketConnection(BaseConnection):
     :type name: string
 
     """
+
     def __init__(self, sock, bufsize=4095, name=None):
         BaseConnection.__init__(self, name)
 
@@ -161,13 +174,12 @@ class SocketConnection(BaseConnection):
         self.opened = False
         self.rxthread = None
         self.sock = sock
-        self.sock.settimeout(0.1)	# for recv
-        self.bufsize=bufsize
-
+        self.sock.settimeout(0.1)  # for recv
+        self.bufsize = bufsize
 
     def open(self):
         self.exit_requested = False
-        self.rxthread = threading.Thread(target=self.rxthread_task)
+        self.rxthread = threading.Thread(target=self.rxthread_task, daemon=True)
         self.rxthread.start()
         self.opened = True
         self.logger.info('Connection opened')
@@ -193,7 +205,6 @@ class SocketConnection(BaseConnection):
             except Exception:
                 self.exit_requested = True
 
-
     def close(self):
         self.exit_requested = True
         self.rxthread.join()
@@ -225,7 +236,6 @@ class SocketConnection(BaseConnection):
             self.rxqueue.get()
 
 
-
 class IsoTPSocketConnection(BaseConnection):
     """
     Sends and receives data through an ISO-TP socket. Makes cleaner code than SocketConnection but offers no additional functionality.
@@ -247,13 +257,14 @@ class IsoTPSocketConnection(BaseConnection):
     :type kwargs: dict
 
     """
+
     def __init__(self, interface, rxid, txid, name=None, tpsock=None, *args, **kwargs):
 
         BaseConnection.__init__(self, name)
 
-        self.interface=interface
-        self.rxid=rxid
-        self.txid=txid
+        self.interface = interface
+        self.rxid = rxid
+        self.txid = txid
         self.rxqueue = queue.Queue()
         self.exit_requested = False
         self.opened = False
@@ -270,11 +281,10 @@ class IsoTPSocketConnection(BaseConnection):
         else:
             self.tpsock = tpsock
 
-
     def open(self):
         self.tpsock.bind(self.interface, rxid=self.rxid, txid=self.txid, *self.tpsock_bind_args, **self.tpsock_bind_kwargs)
         self.exit_requested = False
-        self.rxthread = threading.Thread(target=self.rxthread_task)
+        self.rxthread = threading.Thread(target=self.rxthread_task, daemon=True)
         self.rxthread.start()
         self.opened = True
         self.logger.info('Connection opened')
@@ -300,7 +310,6 @@ class IsoTPSocketConnection(BaseConnection):
             except Exception:
                 self.exit_requested = True
 
-
     def close(self):
         self.exit_requested = True
         self.rxthread.join()
@@ -339,6 +348,7 @@ class IsoTPConnection(IsoTPSocketConnection):
     """
     pass
 
+
 class QueueConnection(BaseConnection):
     """
     Sends and receives data using 2 Python native queues.
@@ -352,11 +362,12 @@ class QueueConnection(BaseConnection):
     :type name: string
 
     """
+
     def __init__(self, name=None, mtu=4095):
         BaseConnection.__init__(self, name)
 
-        self.fromuserqueue = queue.Queue()	# Client reads from this queue. Other end is simulated
-        self.touserqueue = queue.Queue()	# Client writes to this queue. Other end is simulated
+        self.fromuserqueue = queue.Queue()  # Client reads from this queue. Other end is simulated
+        self.touserqueue = queue.Queue()  # Client writes to this queue. Other end is simulated
         self.opened = False
         self.mtu = mtu
 
@@ -372,13 +383,13 @@ class QueueConnection(BaseConnection):
         self.close()
 
     def is_open(self):
-        return self.opened 
+        return self.opened
 
     def close(self):
         self.empty_rxqueue()
         self.empty_txqueue()
         self.opened = False
-        self.logger.info('Connection closed')	
+        self.logger.info('Connection closed')
 
     def specific_send(self, payload):
         if self.mtu is not None:
@@ -439,20 +450,20 @@ class PythonIsoTpConnection(BaseConnection):
     def __init__(self, isotp_layer, name=None):
         BaseConnection.__init__(self, name)
         self.toIsoTPQueue = queue.Queue()
-        self.fromIsoTPQueue = queue.Queue()	
+        self.fromIsoTPQueue = queue.Queue()
         self.rxthread = None
         self.exit_requested = False
         self.opened = False
         self.isotp_layer = isotp_layer
 
-        assert isinstance(self.isotp_layer, isotp.TransportLayer) , 'isotp_layer must be a valid isotp.TransportLayer '
+        assert isinstance(self.isotp_layer, isotp.TransportLayer), 'isotp_layer must be a valid isotp.TransportLayer '
 
     def open(self, bus=None):
         if bus is not None:
             self.isotp_layer.set_bus(bus)
 
         self.exit_requested = False
-        self.rxthread = threading.Thread(target=self.rxthread_task)
+        self.rxthread = threading.Thread(target=self.rxthread_task, daemon=True)
         self.rxthread.start()
         self.opened = True
         self.logger.info('Connection opened')
@@ -465,16 +476,16 @@ class PythonIsoTpConnection(BaseConnection):
         self.close()
 
     def is_open(self):
-        return self.opened 
+        return self.opened
 
     def close(self):
         self.empty_rxqueue()
         self.empty_txqueue()
-        self.exit_requested=True
+        self.exit_requested = True
         self.rxthread.join()
         self.isotp_layer.reset()
         self.opened = False
-        self.logger.info('Connection closed')	
+        self.logger.info('Connection closed')
 
     def specific_send(self, payload):
         if self.mtu is not None:
@@ -482,7 +493,7 @@ class PythonIsoTpConnection(BaseConnection):
                 self.logger.warning("Truncating payload to be set to a length of %d" % (self.mtu))
                 payload = payload[0:self.mtu]
 
-        self.toIsoTPQueue.put(bytearray(payload)) # isotp.protocol.TransportLayer uses byte array. udsoncan is strict on bytes format
+        self.toIsoTPQueue.put(bytearray(payload))  # isotp.protocol.TransportLayer uses byte array. udsoncan is strict on bytes format
 
     def specific_wait_frame(self, timeout=2):
         if not self.opened:
@@ -503,7 +514,7 @@ class PythonIsoTpConnection(BaseConnection):
                 self.logger.warning("Truncating received payload to a length of %d" % (self.mtu))
                 frame = frame[0:self.mtu]
 
-        return bytes(frame)	# isotp.protocol.TransportLayer uses bytearray. udsoncan is strict on bytes format
+        return bytes(frame)  # isotp.protocol.TransportLayer uses bytearray. udsoncan is strict on bytes format
 
     def empty_rxqueue(self):
         while not self.fromIsoTPQueue.empty():
@@ -511,7 +522,7 @@ class PythonIsoTpConnection(BaseConnection):
 
     def empty_txqueue(self):
         while not self.toIsoTPQueue.empty():
-            self.toIsoTPQueue.get()			
+            self.toIsoTPQueue.get()
 
     def rxthread_task(self):
         while not self.exit_requested:
@@ -530,6 +541,7 @@ class PythonIsoTpConnection(BaseConnection):
                 self.exit_requested = True
                 self.logger.error(str(e))
 
+
 class J2534Connection(BaseConnection):
     """
     Sends and receives data through a J2534 Interface. 
@@ -551,43 +563,44 @@ class J2534Connection(BaseConnection):
     :type kwargs: dict
 
     """
-    def __init__(self, windll, rxid, txid, name=None, debug = False, *args, **kwargs):
+
+    def __init__(self, windll, rxid, txid, name=None, debug=False, *args, **kwargs):
 
         BaseConnection.__init__(self, name)
 
-        #Set up a J2534 interface using the DLL provided
-        self.interface = J2534(windll = windll, rxid = rxid, txid = txid)
+        # Set up a J2534 interface using the DLL provided
+        self.interface = J2534(windll=windll, rxid=rxid, txid=txid)
 
-        #Set the protocol to ISO15765, Baud rate to 500000
+        # Set the protocol to ISO15765, Baud rate to 500000
         self.protocol = Protocol_ID.ISO15765
         self.baudrate = 500000
 
-        #Open the interface (connect to the DLL)
+        # Open the interface (connect to the DLL)
         result, self.devID = self.interface.PassThruOpen()
 
         if debug:
-            result = self.interface.PassThruIoctl(Handle = 0,IoctlID = Ioctl_Flags.TX_IOCTL_SET_DLL_DEBUG_FLAGS, ioctlInput = Ioctl_Flags.TX_IOCTL_DLL_DEBUG_FLAG_J2534_CALLS)
+            result = self.interface.PassThruIoctl(Handle=0, IoctlID=Ioctl_Flags.TX_IOCTL_SET_DLL_DEBUG_FLAGS,
+                                                  ioctlInput=Ioctl_Flags.TX_IOCTL_DLL_DEBUG_FLAG_J2534_CALLS)
 
-        #Get the firmeware and DLL version etc, mainly for debugging output
+        # Get the firmeware and DLL version etc, mainly for debugging output
         self.result, self.firmwareVersion, self.dllVersion, self.apiVersion = self.interface.PassThruReadVersion(self.devID)
-        self.logger.info("J2534 FirmwareVersion: " + str(self.firmwareVersion.value) + ", dllVersoin: " + str(self.dllVersion.value) + ", apiVersion" + str(self.apiVersion.value))
+        self.logger.info("J2534 FirmwareVersion: " + str(self.firmwareVersion.value) + ", dllVersoin: " +
+                         str(self.dllVersion.value) + ", apiVersion" + str(self.apiVersion.value))
 
-        #get the channel ID of the interface (used for subsequent communication)
+        # get the channel ID of the interface (used for subsequent communication)
         self.result, self.channelID = self.interface.PassThruConnect(self.devID, self.protocol.value, self.baudrate)
 
-        #Set the filters and clear the read buffer (filters will be set based on tx/rxids)
+        # Set the filters and clear the read buffer (filters will be set based on tx/rxids)
         self.result = self.interface.PassThruStartMsgFilter(self.channelID, self.protocol.value)
         self.result = self.interface.PassThruIoctl(self.channelID, Ioctl_ID.CLEAR_RX_BUFFER)
 
-
         self.rxqueue = queue.Queue()
         self.exit_requested = False
         self.opened = False
 
     def open(self):
         self.exit_requested = False
-        self.rxthread = threading.Thread(target=self.rxthread_task)
-        self.rxthread.daemon = True
+        self.rxthread = threading.Thread(target=self.rxthread_task, daemon=True)
         self.rxthread.start()
         self.opened = True
         self.logger.info('J2534 Connection opened')
@@ -603,19 +616,18 @@ class J2534Connection(BaseConnection):
         return self.opened
 
     def rxthread_task(self):
-        
+
         while not self.exit_requested:
-            
+
             try:
                 result, data, numMessages = self.interface.PassThruReadMsgs(self.channelID, self.protocol.value, 1, 1)
-                
+
                 if data is not None:
                     self.rxqueue.put(data)
             except Exception:
                 self.logger.critical("Exiting J2534 rx thread")
                 self.exit_requested = True
 
-
     def close(self):
         self.exit_requested = True
         self.rxthread.join()
@@ -647,23 +659,24 @@ class J2534Connection(BaseConnection):
         while not self.rxqueue.empty():
             self.rxqueue.get()
 
+
 class FakeConnection(BaseConnection):
     """
     Sends and receives STATIC data defined in a local dict. 
     Used so that an application can be tested without a live can network
     """
-    def __init__(self, name=None, debug = False, *args, **kwargs):
+
+    def __init__(self, name=None, debug=False, *args, **kwargs):
 
         BaseConnection.__init__(self, name)
 
         self.rxqueue = queue.Queue()
-        
+
         self.exit_requested = False
         self.opened = False
 
         self.ResponseData = {b'\x10\x03': b'\x50\x03\x12\x23\x34\x45',
-                b'\x22\xf1\x90\xf1\x89\xf1\x91\xf8\x06\xf1\xa3': b'\x22\xf1\x90\xf1\x89\xf1\x91\xf8\x06\xf1\xa3'}
-
+                             b'\x22\xf1\x90\xf1\x89\xf1\x91\xf8\x06\xf1\xa3': b'\x22\xf1\x90\xf1\x89\xf1\x91\xf8\x06\xf1\xa3'}
 
     def open(self):
         self.opened = True
@@ -709,5 +722,72 @@ class FakeConnection(BaseConnection):
         while not self.rxqueue.empty():
             self.rxqueue.get()
 
-    
 
+class SyncAioIsotpConnection(BaseConnection):
+    """
+    A wrapper for aioisotp sync variant
+
+    `aioisotp <https://github.com/christiansandberg/aioisotp>`_ must be installed in order to use this connection.
+
+    See an :ref:`example<example_using_aioisotp>`
+
+    :param rxid: The reception CAN id
+    :type rxid: int
+
+    :param txid: The transmission CAN id
+    :type txid: int
+
+    :param name: This name is included in the logger name so that its output can be redirected. The logger name will be ``Connection[<name>]``
+    :type name: string
+
+    :param args: Optional parameters list passed to aioisotp binding method.
+    :type args: list
+
+    :param kwargs: Optional parameters dictionary passed to aioisotp binding method.
+    :type kwargs: dict
+    """
+
+    def __init__(self, rx_id, tx_id, name=None, *args, **kwargs):
+        BaseConnection.__init__(self, name)
+        self.network = SyncISOTPNetwork(*args, **kwargs)
+        self.opened = False
+        self.rx_id = rx_id
+        self.tx_id = tx_id
+
+    def specific_send(self, payload):
+        self.conn.send(payload)
+
+    def specific_wait_frame(self, timeout=2):
+        if not self.opened:
+            raise RuntimeError("Connection is not open")
+
+        frame = self.conn.recv(timeout)
+
+        if frame is None and timeout:
+            raise TimeoutException("Did not received frame in time (timeout=%s sec)" % timeout)
+
+        return frame
+
+    def open(self):
+        self.network.open()
+        self.conn = self.network.create_sync_connection(self.rx_id, self.tx_id)
+        self.opened = True
+        self.logger.info("Connection opened")
+        return self
+
+    def close(self):
+        self.network.close()
+        self.opened = False
+        self.logger.info("Connection closed")
+
+    def empty_rxqueue(self):
+        self.conn.empty()
+
+    def is_open(self):
+        return self.opened
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.close()
